/**
 * Create a `StarryNight` that can highlight things based on the given
 * `grammars`.
 * This is async to facilitate async loading and registering, which is currently
 * only used for WASM.
 *
 * @param {Array<Grammar>} grammars
 *   Grammars to support.
 * @param {Options} [options]
 *   Configuration (optional).
 */
export function createStarryNight(
  grammars: Array<Grammar>,
  options?: Options | undefined
): Promise<{
  flagToScope: (flag: string) => string | undefined
  scopes: () => Array<string>
  register: (grammars: Array<Grammar>) => Promise<void>
  highlight: (value: string, scope: string) => Root
}>
export type Root = import('hast').Root
export type Grammar = GrammarFields & PatternFields
export type Rule =
  | RuleInclude
  | RuleName
  | RulePatterns
  | RuleDefinitionBeginEnd
  | RuleDefinitionBeginWhile
  | RuleDefinitionMatch
  | RuleDefinitionCaptures
export type RuleInclude = RuleIncludeFields
export type RuleName = RuleNameFields
export type RulePatterns = PatternFields
export type RuleDefinitionBeginEnd = RuleDefinitionSharedFields &
  RuleDefinitionBeginEndFields
export type RuleDefinitionBeginWhile = RuleDefinitionSharedFields &
  RuleDefinitionBeginWhileFields
export type RuleDefinitionMatch = RuleDefinitionSharedFields &
  RuleDefinitionMatchFields
export type RuleDefinitionCaptures = Omit<
  RuleDefinitionSharedFields,
  'captures'
> &
  RuleDefinitionCapturesFields
export type GrammarFields = {
  scopeName: string
  names: Array<string>
  extensions: Array<string>
}
export type PatternFields = {
  patterns: Array<Rule>
  repository?: Record<string, Rule> | undefined
  injections?: Record<string, Rule> | undefined
}
export type RuleNameFields = {
  name: string
}
export type RuleIncludeFields = {
  include: string
}
export type RuleDefinitionSharedFields = {
  name?: string | undefined
  contentName?: string | undefined
  captures?: Captures | undefined
  beginCaptures?: Captures | undefined
  endCaptures?: Captures | undefined
  applyEndPatternLast?: boolean | undefined
}
export type RuleDefinitionBeginEndFields = {
  begin: string
  end: string
}
export type RuleDefinitionBeginWhileFields = {
  begin: string
  while: string
}
export type RuleDefinitionMatchFields = {
  match: string
}
export type RuleDefinitionCapturesFields = {
  captures: Captures
}
export type Captures = Record<number | string, Rule>
/**
 * Configuration (optional).
 */
export type Options = {
  /**
   * Get a URL to the oniguruma WASM, typically used in browsers.
   */
  getOnigurumaUrlFetch?: GetOnigurumaUrl | undefined
  /**
   * Get a URL to the oniguruma WASM, typically used in Node.js.
   */
  getOnigurumaUrlFs?: GetOnigurumaUrl | undefined
}
/**
 * Get a URL to the oniguruma WASM.
 *
 * > 👉 **Note**: this must currently result in a version 1 URL of
 * > `onig.wasm` from `vscode-oniguruma`.
 *
 * > ⚠️ **Danger**: when you use this functionality, your project might break at
 * > any time (when reinstalling dependencies), except when you make sure that
 * > the WASM binary you load manually is what our internally used
 * > `vscode-oniguruma` dependency expects.
 * > To solve this, you could for example use an npm script called `dependencies`
 * > (which runs everytime `node_modules` is changed) which copies
 * > `vscode-oniguruma/release/onig.wasm` to the place you want to host it.
 */
export type GetOnigurumaUrl = () => URL | Promise<URL>
